#!/bin/bash
VER="1.0"
########################################################################################
# Tur-AutoNuke by Turranius ( http://www.grandis.nu/glftpd/ ) turranius@hotmail.com    #
#-[ Introduction ]---------------------------------------------------------------------#
#                                                                                      #
# This is an autonuking script for empty folders and incompleted uploads.              #
# It can also be set to clean up any incomplete symlinks left behind if a release is   #
# removed.                                                                             #
#                                                                                      #
#-[ Installation ]---------------------------------------------------------------------#
#                                                                                      #
# Copy this script to /glftpd/bin/ or where ever really. Make it executable.           #
# Setup all the options below. Be careful unless you want the whole site nuked.        #
# !! Before running it the first time, use the variable test (tur-autonuke.sh test) !! #
#                                                                                      #
# today=      Todays dated structure. I included today so you can add it to dir, thus  #
#             checking in todays dated folder too. You should not need to change this  #
#             unless your dated dir structure is different then 0527 (May 27th).       #
#             Just leave this as it is if you do not have any dated folders.           #
#                                                                                      #
# yesterday=  Same as above. Included this one so you can also autonuke in yesterdays  #
#             dated folder. Shouldnt need to change this one either.                   #
#                                                                                      #
# GLROOT=     The root folder of your glftpd setup. Usually /glftpd                    #
#                                                                                      #
# GLCONF=     Im reading glftpd.conf to check how your nuked folders look, and also    #
#             use it when I nuke (well, the nuker prog uses it).                       #
#             Usually /etc/glftpd.conf                                                 #
#                                                                                      #
# TEMPDIR=    A temporary folder where I can store stuff.                              #
#                                                                                      #
# SYMLINKNAME = Now this is an important one. Make sure you get this one right.        #
#               What is the prefix that your zipscript adds before a link to specify   #
#               that this is an incompleted release? For zipscript-c, its              #
#               [incomplete]- so that is the default. If you do not set this one right #
#               it will either not find any incompletes at all, or it will think that  #
#               all folders are incomplete. Do not forget any possible - or something. #
#                                                                                      #
# LOG=        What file do I log to, incase of a nuke or cleanup of a symlink.         #
#             Note, this is not glftpd.log. The NUKEPROG below will do bot announce    #
#             like normal nukes.                                                       #
#                                                                                      #
# NUKEPROG=   The executable file used to nuke folders. This one comes with glftpd and #
#             should already be in your bin dir. Try running it from shell.            #
#                                                                                      #
# TIMEEMPTY=  How old, in minutes, must an empty folder be, before I nuke it?          #
#                                                                                      #
# TIMEINCOM=  How old, in minutes, must an incompleted release be, before I nuke it?   #
#             This one should be a lot larger then TIMEEMPTY, unless you want to nuke  #
#             stuff that people are in the process of uploading.                       #
#                                                                                      #
# EMPTYMULTIPLIER= Whats the multiplier to nuke empty folders? This will be used when  #
#                  nuking empty folders.                                               #
#                                                                                      #
# INCOMMULTIPLIER= Whats the multiplier to nuke incompleted releases?                  #
#                                                                                      #
# NUKEUSER=   Which user should I use when nuking? Make sure this user exists and has  #
#             flag A for nuking. Should just contain the users exact username.         #
#                                                                                      #
# CLEANSYMLINKS=   TRUE/FALSE. If I find a symlink, but the folder it points to is not #
#                  there, should I remove the symlink?                                 #
#                                                                                      #
# CLEANMESSAGE=    If your zipscript also creates a .message file in the root of the   #
#                  section with all incompletes (jzipscript does this), then set this  #
#                  to TRUE too. It will then remove the incomplete line from the       #
#                  .message file when it nukes it. Thanks soehest for the idea.        #
#                                                                                      #
# EXCLUDES=    When it comes to incompleted releases, should I look for something in   #
#              the foldername and exclude it? Usually, NFO fixes and similar counts as #
#              an incompleted release, even though they are not, and thats why we have #
#              nfo.fix in here as default. Separate excludes with a space.             #
#              Excludes are not case sensitive (nfo.fix is the same as NFO.Fix etc).   #
#              It will clean up the symlink to the folder, even though its excluded.   #
#              Better add your predir name here to, if you have one.                   #
#                                                                                      #
# JUMPON=      Another important one. If you have a release, say Saggebulla and this   #
#              one contans two CD's, and CD1 is incompleted, should I jump up one      #
#              folder and nuke the Saggebulla folder?                                  #
#              In short, as default, there is a lot of CD's here. If it finds one of   #
#              these names incompleted, it will jump up one step to the parent folder  #
#              and nuke that one instead. I like it when it nukes a whole release if   #
#              any of the CD's in it are incomplete.                                   #
#              If you set this to "", it will just nuke the actual CD instead.         #
#              Any subfolder can be added here. Like VobSub etc.                       #
#              This one IS case sensitive.                                             #
#                                                                                      #
# DIRS=        Which folders should I look in, relative to GLROOT? Separate them with  #
#              a space. As you see from the default, I utilize $today and $yesterday   #
#              to also look in todays and yesterdays dated folder in 0DAYS.            #
#                                                                                      #
# !! Before running it the first time, use the variable test (tur-autonuke.sh test) !! #
#                                                                                      #
#-[ Testing ]--------------------------------------------------------------------------#
#                                                                                      #
# To test this baby, create one empty folder inside any of the defined DIRS.           #
# Create another one and upload a .sfv in it so that it creates an incomplete symlink. #
# Run the script with the test parameter, and it should not say anything about the     #
# empty folder, until TIMEEMPTY minutes have passed.                                   #
# For the incompleted folder, it should find the symlink instantly, but not say        #
# anything else, until TIMEINCIM minutes have passed.                                  #
#                                                                                      #
#-[ irc ]------------------------------------------------------------------------------#
# Contact Turranius on Efnet. ( turranius / turran / turr|away / turr|work )           #
#                                                                                      #
#-[ Settings ]-------------------------------------------------------------------------#

today="$( date +%m%d )"
yesterday="$( date --date "yesterday" +%m%d )"

GLROOT="/glftpd"
GLCONF="/etc/glftpd.conf"
TEMPDIR="/glftpd/tmp"
SYMLINKNAME="[incomplete]-"
LOG="/glftpd/ftp-data/logs/tur-autonuke.log"
NUKEPROG="/glftpd/bin/nuker"
TIMEEMPTY="15"
TIMEINCOM="200"
EMPTYMULTIPLIER="3"
INCOMMULTIPLIER="3"
NUKEUSER="turranius"
CLEANSYMLINKS="TRUE"
CLEANMESSAGE="FALSE"
EXCLUDES="nfo.fix GROUPS"

JUMPON="CD1 CD2 CD3 CD4 CD5 CD6 CD7 Cd1 Cd2 Cd3 Cd4 Cd5 Cd6 Cd7"

DIRS="/site/VCD /site/XXX /site/SVCD /site/ISO-UTILS /site/DIVX /site/0DAYS/$today /site/0DAYS/$yesterday"


#################################################################
# Yadda, no changes below here, rip off balls etc etc.          #
#################################################################

## Check date for the logfile.
DATENOW="$(date +%D" - "%T)"

## Nuked dir syntax. Dont want to nuke those again.
NUKES="$( grep nukedir_style $GLCONF | egrep -v NiA | egrep -iv '#' | cut -f2-2 | awk -F"%" -F"-" '{print $1}' )"

if [ -e $LOG ]; then
  foundit=yes
else
  touch $LOG
  chmod 777 $LOG
fi

if [ "$1" = "test" ]; then
  echo ""
  echo "-[ Checking empty folders ]-"
fi

for folders in $DIRS; do
  if [ -e $GLROOT$folders ]; then
    cd $GLROOT$folders
  else
    echo "I can not find $GLROOT$folders - I'll quit here while you fix your mistake."
    echo "Check the DIRS= settings."
    exit 0
  fi

  if [ "$1" = "test" ]; then
    echo ""
    echo "--Entering $folders"
  fi
  for found in `find $GLROOT$folders -empty -type d -mmin +$TIMEEMPTY -maxdepth 1 -print | grep -vw "$NUKES" | grep -vx $GLROOT$folders | cut -b9-`; do
   if [ "$1" = "test" ]; then
     echo "Folder: $folders  Other: $found  Empty more then $TIMEEMPTY minutes. Nuking it (test)."
   fi
   if [ "$1" != "test" ]; then
     $NUKEPROG -r $GLCONF -N $NUKEUSER -n {/$found} $EMPTYMULTIPLIER -Auto- Empty for more than $TIMEEMPTY minutes.
     echo "$DATENOW Nuke: /$found - Empty after $TIMEEMPTY minutes. $EMPTYMULTIPLIER Nuke." >> $LOG
   fi
  done
done

if [ "$1" = "test" ]; then
  echo ""
  echo "-[ Checking incompletes ]-"
fi

## Look for incompletes.
for folders in $DIRS; do
  if [ -e $GLROOT$folders ]; then
    cd $GLROOT$folders
  else
    echo "I can not find $GLROOT$folders - Quitting until you fix this."
    exit 0
  fi

  if [ "$1" = "test" ]; then
    echo ""
    echo "--- Entering $folders"
  fi
  LIST="$( ls -m )"

  for dirs in $LIST; do
    VERIFY="$( echo $dirs | grep -F $SYMLINKNAME )"
    if [ "$VERIFY" != "" ]; then
      VERIFY="$( echo $VERIFY | tr -d ',' )"
      if [ "$1" = "test" ]; then
        echo "Found incomplete link: $VERIFY"
      fi
      CHECK="$( ls -l $GLROOT$folders | grep -F $VERIFY )"
      CHECK="$( echo $CHECK | awk -F ">" '{print $2}' | tr -s '/' ' ' )"
      for crap in $CHECK; do
        LAST="$REALNAME"
        REALNAME="$crap"
      done
      for jump in $JUMPON; do
        if [ "$REALNAME" = "$jump" ]; then
          REALNAME="$LAST"
        fi
      done

      if [ -e "$REALNAME" ]; then
    for folder in `find $REALNAME -type d -mmin +$TIMEINCOM -maxdepth 1 -print | grep -vw "$NUKES" | grep -vx $REALNAME | cut -b9-`; do
        if [ "$REALNAME" != "$LAST" ]; then
          EXCLUDE="NO"
          for excludes in $EXCLUDES; do
            VERIFY2="$( echo $REALNAME | grep -i $excludes )"
            if [ "$VERIFY2" != "" ]; then
              EXCLUDE="YES"
            fi
          done

          if [ "$EXCLUDE" != "YES" ]; then
            if [ "$1" = "test" ]; then
              echo "$REALNAME is older then $TIMEINCOM minutes. Nuking it (test)."
            fi

            if [ "$CLEANMESSAGE" = "TRUE" ]; then
              if [ -e $GLROOT/$folders/.message ]; then
                if [ "$1" = "test" ]; then
                  CHECKMESSAGE="$( cat $GLROOT/$folders/.message | grep -w $REALNAME )"
                  echo "Removing release from .message (test, line follows below:)"
                  echo "$CHECKMESSAGE"
                else
                  cat $GLROOT/$folders/.message | grep -v "$REALNAME" > $TEMPDIR/message.tmp
                  mv -f $TEMPDIR/message.tmp $GLROOT/$folders/.message
                  echo "Actually removed it from .message"
                fi
              else
                
                echo "CLEANMESSAGE is TRUE, but no .message found in $GLROOT/$folders."
              fi
            fi
              
            if [ "$1" != "test" ]; then
              $NUKEPROG -r $GLCONF -N $NUKEUSER -n {$folders/$REALNAME} $INCOMMULTIPLIER -Auto- Not completed for $TIMEINCOM minutes.
             echo "$DATENOW Nuke: $folders/$REALNAME - Incomplete after $TIMEINCOM minutes. $INCOMMULTIPLIER Nuke." >> $LOG
            fi
            LAST="$REALNAME"
          else
            if [ "$1" = "test" ]; then
              if [ "$SAIDIT" != "$REALNAME" ]; then
                echo "$REALNAME is excluded".
                SAIDIT="$REALNAME"
              fi
            fi
          fi
          if [ "$CLEANSYMLINKS" = "TRUE" ]; then
            if [ "$1" != "test" ]; then
              rm -rf $VERIFY
              echo "$DATENOW Dele: Removing symlink $VERIFY" >> $LOG
            fi
            if [ "$1" = "test" ]; then
              if [ "$SAIDIT2" != "$REALNAME" ]; then
                echo "Removing symlink: $VERIFY (test)"
                echo ""
                SAIDIT2="$REALNAME"
              fi
            fi
          fi
        fi
  done
      else
        if [ "$1" = "test" ]; then
          echo "$REALNAME does NOT exist anymore"
        fi
        if [ "$CLEANSYMLINKS" = "TRUE" ]; then
          if [ "$1" != "test" ]; then
            rm -rf $VERIFY
            echo "$DATENOW Dele: Removing symlink $VERIFY" >> $LOG
          fi
          if [ "$1" = "test" ]; then
            echo "Removing symlink to $VERIFY"
          fi
        fi
      fi
    fi
  done
done

exit 0
